@@ -4,7 +4,7 @@ emepy
 Eigenmode Expansion Python
 """
 
-__version__ = "1.1.0"
+__version__ = "1.2.1"
 __author__ = "Ian Hammond"
 __credits__ = "BYU CamachoLab"
 
@@ -14,9 +14,6 @@ from emepy.mode import *
 from emepy.fd import *
 from emepy.models import *
 from emepy.source import *
-
-# from emepy.geometries import *
-from emepy.gpu import *
 from emepy.ann import *
 from emepy.monitors import *
 from emepy.lumerical import *
