@@ -130,7 +130,9 @@ class EigenMode(object):
 
         return self._inner_product(self, mode2)
 
-    def check_spurious(self, threshold_power: float = 0.05, threshold_neff: float = 0.9) -> bool:
+    def check_spurious(
+        self, threshold_power: float = 0.05, threshold_neff: float = 0.9
+    ) -> bool:
         """Takes in a mode and determine whether the mode is likely spurious based on the ratio of confined to not confined power
 
         Parameters
@@ -209,8 +211,12 @@ class Mode1D(EigenMode):
         self.Ey = Ey if Ey is not None else self.Hx * 0
         self.Ez = Ez if Ez is not None else self.Hx * 0
         self.n = n
-        self.H = np.sqrt(np.abs(self.Hx) ** 2 + np.abs(self.Hy) ** 2 + np.abs(self.Hz) ** 2)
-        self.E = np.sqrt(np.abs(self.Ex) ** 2 + np.abs(self.Ey) ** 2 + np.abs(self.Ez) ** 2)
+        self.H = np.sqrt(
+            np.abs(self.Hx) ** 2 + np.abs(self.Hy) ** 2 + np.abs(self.Hz) ** 2
+        )
+        self.E = np.sqrt(
+            np.abs(self.Ex) ** 2 + np.abs(self.Ey) ** 2 + np.abs(self.Ez) ** 2
+        )
 
     def plot(self, operation: str = "Real", normalize: bool = True) -> None:
         """Plots the fields in the mode using pyplot. Should call plt.figure() before and plt.show() or plt.savefig() after
@@ -224,13 +230,21 @@ class Mode1D(EigenMode):
         """
 
         temp = (
-            self / max([np.abs(np.real(np.amax(i))) for i in [self.Ex, self.Ey, self.Ez, self.Hx, self.Hy, self.Hz]])
+            self
+            / max(
+                [
+                    np.abs(np.real(np.amax(i)))
+                    for i in [self.Ex, self.Ey, self.Ez, self.Hx, self.Hy, self.Hz]
+                ]
+            )
             if normalize
             else self / 1
         )
 
         # Parse operation
-        op_name = operation.__name__ if hasattr(operation, "__name__") else str(operation)
+        op_name = (
+            operation.__name__ if hasattr(operation, "__name__") else str(operation)
+        )
         if operation == "Imaginary":
             operation = lambda a: np.imag(a)
         elif operation == "Abs":
@@ -240,7 +254,9 @@ class Mode1D(EigenMode):
         elif operation == "Real":
             operation = lambda a: np.real(a)
         try:
-            t = self.change_fields(deepcopy(temp), deepcopy(temp), lambda a, b: operation(b))
+            t = self.change_fields(
+                deepcopy(temp), deepcopy(temp), lambda a, b: operation(b)
+            )
             Hx, Hy, Hz, Ex, Ey, Ez = [t.Hx, t.Hy, t.Hz, t.Ex, t.Ey, t.Ez]
         except Exception as e:
             print(e)
@@ -257,7 +273,9 @@ class Mode1D(EigenMode):
             plt.ylabel("{}({})".format(op_name, fields[i]))
         plt.tight_layout()
 
-    def _inner_product(self, mode1: EigenMode, mode2: EigenMode, mask: "np.ndarray" = None) -> float:
+    def _inner_product(
+        self, mode1: EigenMode, mode2: EigenMode, mask: "np.ndarray" = None
+    ) -> float:
         """Helper function that takes the inner product between Modes mode1 and mode2
 
         Parameters
@@ -308,7 +326,9 @@ class Mode1D(EigenMode):
         kernel = np.ones(num_pixels + 1)
         mask = convolve(mask, kernel, "same")
         mask = np.where(mask > 0, 1, 0)
-        ratio = self._inner_product(self, self, mask=mask) / self._inner_product(self, self, mask=None)
+        ratio = self._inner_product(self, self, mask=mask) / self._inner_product(
+            self, self, mask=None
+        )
         return ratio
 
     def zero_phase(self) -> None:
@@ -320,7 +340,9 @@ class Mode1D(EigenMode):
         if (np.sum(np.real(self.Hy))) < 0:
             self *= -1
 
-    def plot_material(self, operation: Callable[["np.ndarray"], "np.ndarray"] = np.real) -> None:
+    def plot_material(
+        self, operation: Callable[["np.ndarray"], "np.ndarray"] = np.real
+    ) -> None:
         """Plots the index of refraction profile"""
         plt.plot(self.x, operation(self.n))
         plt.title("Index of Refraction")
@@ -384,10 +406,16 @@ class Mode(EigenMode):
         self.Ey = Ey if Ey is not None else self.Hx * 0
         self.Ez = Ez if Ez is not None else self.Hx * 0
         self.n = n
-        self.H = np.sqrt(np.abs(self.Hx) ** 2 + np.abs(self.Hy) ** 2 + np.abs(self.Hz) ** 2)
-        self.E = np.sqrt(np.abs(self.Ex) ** 2 + np.abs(self.Ey) ** 2 + np.abs(self.Ez) ** 2)
+        self.H = np.sqrt(
+            np.abs(self.Hx) ** 2 + np.abs(self.Hy) ** 2 + np.abs(self.Hz) ** 2
+        )
+        self.E = np.sqrt(
+            np.abs(self.Ex) ** 2 + np.abs(self.Ey) ** 2 + np.abs(self.Ez) ** 2
+        )
 
-    def plot(self, operation: str = "Real", colorbar: bool = True, normalize: bool = True) -> None:
+    def plot(
+        self, operation: str = "Real", colorbar: bool = True, normalize: bool = True
+    ) -> None:
         """Plots the fields in the mode using pyplot. Should call plt.figure() before and plt.show() or plt.savefig() after
 
         Parameters
@@ -401,13 +429,21 @@ class Mode(EigenMode):
         """
 
         temp = (
-            self / max([np.abs(np.real(np.amax(i))) for i in [self.Ex, self.Ey, self.Ez, self.Hx, self.Hy, self.Hz]])
+            self
+            / max(
+                [
+                    np.abs(np.real(np.amax(i)))
+                    for i in [self.Ex, self.Ey, self.Ez, self.Hx, self.Hy, self.Hz]
+                ]
+            )
             if normalize
             else self / 1
         )
 
         # Parse operation
-        op_name = operation.__name__ if hasattr(operation, "__name__") else str(operation)
+        op_name = (
+            operation.__name__ if hasattr(operation, "__name__") else str(operation)
+        )
         if operation == "Imaginary":
             operation = lambda a: np.imag(a)
         elif operation == "Abs":
@@ -417,7 +453,9 @@ class Mode(EigenMode):
         elif operation == "Real":
             operation = lambda a: np.real(a)
         try:
-            t = self.change_fields(deepcopy(temp), deepcopy(temp), lambda a, b: operation(b))
+            t = self.change_fields(
+                deepcopy(temp), deepcopy(temp), lambda a, b: operation(b)
+            )
             Hx, Hy, Hz, Ex, Ey, Ez = [t.Hx, t.Hy, t.Hz, t.Ex, t.Ey, t.Ez]
         except Exception as e:
             print(e)
@@ -428,18 +466,20 @@ class Mode(EigenMode):
         # Plot fields
         fields = ["Hx", "Hy", "Hz", "Ex", "Ey", "Ez"]
         for i, field in enumerate([Hx, Hy, Hz, Ex, Ey, Ez]):
-            plt.subplot(2, 3, i + 1, adjustable="box", aspect=field.shape[0] / field.shape[1])
+            plt.subplot(
+                2, 3, i + 1, adjustable="box", aspect=field.shape[0] / field.shape[1]
+            )
             v = max(abs(field.min()), abs(field.max()))
             plt.imshow(
-                field.T,
+                np.rot90(field),
                 cmap="RdBu",
                 vmin=-v,
                 vmax=v,
                 extent=[
-                    temp.x[0] * 1e6,
-                    temp.x[-1] * 1e6,
-                    temp.y[0] * 1e6,
-                    temp.y[-1] * 1e6,
+                    temp.x[0],
+                    temp.x[-1],
+                    temp.y[0],
+                    temp.y[-1],
                 ],
                 interpolation="none",
             )
@@ -451,7 +491,9 @@ class Mode(EigenMode):
 
         plt.tight_layout()
 
-    def _inner_product(self, mode1: EigenMode, mode2: EigenMode, mask: "np.ndarray" = None) -> float:
+    def _inner_product(
+        self, mode1: EigenMode, mode2: EigenMode, mask: "np.ndarray" = None
+    ) -> float:
         """Helper function that takes the inner product between Modes mode1 and mode2
 
         Parameters
@@ -501,7 +543,9 @@ class Mode(EigenMode):
         kernel = np.ones((num_pixels + 1, num_pixels + 1))
         mask = convolve2d(mask, kernel, "same")
         mask = np.where(mask > 0, 1, 0)
-        ratio = self._inner_product(self, self, mask=mask) / self._inner_product(self, self, mask=None)
+        ratio = self._inner_product(self, self, mask=mask) / self._inner_product(
+            self, self, mask=None
+        )
         return ratio
 
     def zero_phase(self) -> None:
@@ -516,13 +560,14 @@ class Mode(EigenMode):
     def plot_material(self) -> None:
         """Plots the index of refraction profile"""
 
+        plt.figure()
         plt.imshow(
-            np.real(self.n).T,
+            np.real(np.rot90(self.n)),
             extent=[
-                self.x[0] * 1e6,
-                self.x[-1] * 1e6,
-                self.y[0] * 1e6,
-                self.y[-1] * 1e6,
+                self.x[0],
+                self.x[-1],
+                self.y[0],
+                self.y[-1],
             ],
             cmap="Greys",
             interpolation="none",
@@ -557,7 +602,9 @@ class Mode(EigenMode):
             self.Ex,
             self.Ey,
             self.Ez,
-        ) = compute_other_fields_2D(self.neff, self.Hx, self.Hy, self.wl, self.x, self.y, boundary, epsfunc_1)
+        ) = compute_other_fields_2D(
+            self.neff, self.Hx, self.Hy, self.wl, self.x, self.y, boundary, epsfunc_1
+        )
         x_ = (self.x[1:] + self.x[:-1]) / 2.0
         y_ = (self.y[1:] + self.y[:-1]) / 2.0
         self.Ex = interp(self.x, self.y, x_, y_, self.Ex, False)
@@ -565,6 +612,10 @@ class Mode(EigenMode):
         self.Ez = interp(self.x, self.y, x_, y_, self.Ez, False)
         self.x = self.x - self.x[int(len(self.x) / 2)]
         self.y = self.y - self.y[int(len(self.y) / 2)]
-        self.H = np.sqrt(np.abs(self.Hx) ** 2 + np.abs(self.Hy) ** 2 + np.abs(self.Hz) ** 2)
-        self.E = np.sqrt(np.abs(self.Ex) ** 2 + np.abs(self.Ey) ** 2 + np.abs(self.Ez) ** 2)
+        self.H = np.sqrt(
+            np.abs(self.Hx) ** 2 + np.abs(self.Hy) ** 2 + np.abs(self.Hz) ** 2
+        )
+        self.E = np.sqrt(
+            np.abs(self.Ex) ** 2 + np.abs(self.Ey) ** 2 + np.abs(self.Ez) ** 2
+        )
         self.n = np.sqrt(epsfunc_2(self.x, self.y))
