@@ -32,7 +32,16 @@ class Layer(object):
         self.length = length
         self.activated_layers = []
 
-    def activate_layer(self, sources: list = [], start: float = 0.0, period_length: float = 0.0) -> dict:
+    def begin_activate(self):
+        return ModelTools._solve_modes_wrapper, self.mode_solver
+
+    def finish_activate(self, sources: list = [], start: float = 0.0, period_length: float = 0.0, mode_solver=None):
+        self.mode_solver = mode_solver
+        return self.activate_layer(sources, start, period_length, False)
+
+    def activate_layer(
+        self, sources: list = [], start: float = 0.0, period_length: float = 0.0, compute_modes=True
+    ) -> dict:
         """Solves for the modes in the layer and creates an ActivatedLayer object
 
         Parameters
@@ -54,7 +63,8 @@ class Layer(object):
         modes = []
 
         # Solve for modes
-        self.mode_solver.solve()
+        if compute_modes:
+            self.mode_solver.solve()
         for mode in range(self.num_modes):
             modes.append(self.mode_solver.get_mode(mode))
 
@@ -313,6 +323,7 @@ class ActivatedLayer(Model):
         for name in self.right_pins:
             pins.append(Pin(self, name))
 
+        self.pins = pins
         super().__init__(**kwargs, pins=pins)
 
     def normalize_fields(self) -> None:
@@ -478,7 +489,6 @@ class InterfaceMultiMode(Model):
 
         self.layer1 = layer1
         self.layer2 = layer2
-        self.num_ports = layer1.right_ports + layer2.left_ports
         self.left_ports = layer1.right_ports
         self.right_ports = layer2.left_ports
         self.left_pins = ["left" + str(i) for i in range(self.left_ports)]
@@ -492,6 +502,7 @@ class InterfaceMultiMode(Model):
             pins.append(Pin(self, name))
 
         super().__init__(**kwargs, pins=pins)
+        self.num_ports = layer1.right_ports + layer2.left_ports
         self.solve()
 
     def s_parameters(self, freqs: "np.ndarray" = None) -> "np.ndarray":
@@ -743,7 +754,7 @@ class SourceDuplicator(Model):
 class CopyModel(Model):
     """A simple Model that can be used to deep copy any of EMEPy's Models"""
 
-    def __init__(self, model: "Model", **kwargs) -> None:
+    def __init__(self, model: "Model", keep_modes: bool = True, **kwargs) -> None:
         """Creates an instance of CopyModel by deepcopying all the attributes of model
 
         Parameters
@@ -753,13 +764,13 @@ class CopyModel(Model):
         """
 
         self.num_modes = model.num_modes if hasattr(model, "num_modes") else None
-        self.modes = model.modes if hasattr(model, "modes") else []
+        self.modes = model.modes if hasattr(model, "modes") and keep_modes else []
         self.wavelength = model.wavelength if hasattr(model, "wavelength") else None
         self.length = model.length if hasattr(model, "length") else None
-        self.left_ports = model.left_ports if hasattr(model, "left_ports") else []
-        self.right_ports = model.right_ports if hasattr(model, "right_ports") else []
+        self.left_ports = model.left_ports if hasattr(model, "left_ports") else 0
+        self.right_ports = model.right_ports if hasattr(model, "right_ports") else 0
 
-        self.S0 = ModelTools.make_copy_model(model.S0) if hasattr(model, "S0") else None
+        self.S0 = ModelTools.make_copy_model(model.S0, keep_modes=False) if hasattr(model, "S0") else None
         self.nk = model.nk if hasattr(model, "nk") else []
         self.pk = model.pk if hasattr(model, "pk") else []
         self.pins = self.copy_pins(model.pins) if hasattr(model, "pins") else []
@@ -917,7 +928,8 @@ class ModelTools(object):
     def _prop_all(*args) -> "Model":
         """Given an arbitrary amount of simphony models as inputs, cascades them and returns the cascaded network"""
 
-        layers = [ModelTools.make_copy_model(a) for a in args if a is not None]
+        layers = [ModelTools.make_copy_model(a, keep_modes=False) for a in args if a is not None]
+
         temp_s = layers[0]
         for s in layers[1:]:
             Subcircuit.clear_scache()
@@ -934,7 +946,7 @@ class ModelTools(object):
             temp_s = temp_s.circuit.to_subcircuit()
             temp_s.s_params = temp_s.s_parameters([0])
 
-        return temp_s
+        return ModelTools.make_copy_model(temp_s, keep_modes=False)
 
     @staticmethod
     def periodic_duplicate_format(model: "Model", start: float, end: float) -> "Model":
@@ -993,7 +1005,7 @@ class ModelTools(object):
         return model
 
     @staticmethod
-    def make_copy_model(model: Model) -> CopyModel:
+    def make_copy_model(model: Model, keep_modes=True) -> CopyModel:
         """Takes an EMEPy model (inheriting a simphony model) and deepcopies it
 
         Parameters
@@ -1006,11 +1018,11 @@ class ModelTools(object):
         CopyModel
             the deepcopied model
         """
-        new_model = CopyModel(model) if model is not None else None
+        new_model = CopyModel(model, keep_modes) if model is not None else None
         return new_model
 
     @staticmethod
-    def compute(model, pin_values: "dict", freq: "float") -> dict:
+    def compute(model, pin_values: "dict", freq: "float" = 0) -> dict:
         """Takes a dictionary mapping each pin name to a coefficent and multiplies by the S matrix
 
         Parameters
@@ -1034,3 +1046,17 @@ class ModelTools(object):
         matrix = model.s_parameters(np.array([freq]))
         output = np.matmul(matrix, cfs)[0]
         return dict(zip(pin_names, output))
+
+    # Define tasks
+    @staticmethod
+    def layers_task(l, r, interface_type):
+        return l, ModelTools._prop_all(l, interface_type(l, r))
+
+    @staticmethod
+    def _prop_all_wrapper(arg_list, result_list):
+        return ModelTools._prop_all(*arg_list), result_list
+
+    @staticmethod
+    def _solve_modes_wrapper(mode_solver):
+        mode_solver.solve()
+        return mode_solver
