@@ -2,8 +2,8 @@ import numpy as np
 import numpy
 from matplotlib import pyplot as plt
 import matplotlib
-from copy import deepcopy
 from scipy.interpolate import griddata
+plt.rcParams.update({'figure.max_open_warning': 0})
 
 
 class Monitor(object):
@@ -20,6 +20,8 @@ class Monitor(object):
         grid_z: np.array = None,
         location: float = None,
         sources: list = [],
+        adjoint_n: bool = True,
+        total_length: float = 0.0,
     ) -> None:
         """Monitor class constructor0
 
@@ -43,6 +45,8 @@ class Monitor(object):
             the location in z if the monitor represents "xy" axes (default: None)
         sources : list[Source]
             sources to use for the monitor (default:[])
+        adjoint_n : bool
+            if true will use the "continuous" n used for adjoint
         """
 
         # Ensure z range is in proper format
@@ -56,7 +60,7 @@ class Monitor(object):
                 raise Exception(
                     "z_range should be a tuple or list of the form (start, end)"
                     " representing the range of the z values to extract where start "
-                    "and end are floats such as (0, 1e-6) for a 1 µm range"
+                    "and end are floats such as (0, 1) for a 1 µm range"
                 ) from e
 
         # Check axes
@@ -75,6 +79,7 @@ class Monitor(object):
             )
 
         # Set parameters globally
+        self.adjoint_n = adjoint_n
         self.dimensions = dimensions
         self.field = np.zeros(dimensions).astype(complex)
         self.lengths = grid_z.tolist()
@@ -87,6 +92,8 @@ class Monitor(object):
         self.grid_y = grid_y
         self.grid_z = grid_z
         self.location = location
+        self.total_length = total_length
+        self.xy_monitors = []
 
     def reset_monitor(self) -> None:
         """Resets the fields in the monitor"""
@@ -171,7 +178,7 @@ class Monitor(object):
                 raise Exception(
                     "z_range should be a tuple or list of the form (start, end)"
                     "representing the range of the z values to extract where start"
-                    "and end are floats such as (0, 1e-6) for a 1 µm range"
+                    "and end are floats such as (0, 1) for a 1 µm range"
                 ) from e
 
             # Get start and end
@@ -324,20 +331,33 @@ class Monitor(object):
 
         return grid_field
 
-    def get_source_visual(self, shape) -> "numpy.ndarray":
+    def get_source_visual(self, min, max) -> "numpy.ndarray":
         """Returns a mask with lines indicating where a source is"""
-        array = np.zeros(shape)
-        l = self.dimensions[-1] // 200
+        srcs = []
         if self.right_source:
-            array[:, -l:] = 1
+            srcx = [self.total_length, self.total_length]
+            srcy = [min, max]
+            srcs.append((srcx, srcy))
         if self.left_source:
-            array[:, :l] = 1
+            srcx = [0, 0]
+            srcy = [min, max]
+            srcs.append((srcx, srcy))
         for source in self.sources:
-            difference_start = lambda list_value: abs(list_value - source.z)
-            i = self.lengths.index(min(self.lengths, key=difference_start))
-            array[:, i - l // 2 : i + l // 2] = 1
+            srcx = [source.z, source.z]
+            srcy = [min, max]
+            srcs.append((srcx, srcy))
 
-        return array
+        return srcs
+
+    def get_xy_monitor_visual(self, min, max) -> "numpy.ndarray":
+        """Returns a mask with lines indicating where a source is"""
+        xy_monitors = []
+        for monitor in self.xy_monitors:
+            srcx = [monitor.location, monitor.location]
+            srcy = [min, max]
+            xy_monitors.append((srcx, srcy))
+
+        return xy_monitors
 
     def visualize(
         self,
@@ -348,6 +368,7 @@ class Monitor(object):
         z_range: tuple = None,
         show_geometry: bool = True,
         show_sources: bool = True,
+        show_xy_monitors: bool = False,
     ) -> "matplotlib.image.AxesImage":
         """Creates a matplotlib axis displaying the provides field component
 
@@ -410,52 +431,54 @@ class Monitor(object):
 
         # Create plots
         if axes in ["xz", "zx", "yz", "zy"]:
-            if ax:
-                if show_geometry:
-                    ax.imshow(
-                        np.real(n),
-                        extent=[np.real(zn[0]), np.real(zn[-1]), np.real(yn[0]), np.real(yn[-1])],
-                        cmap=cmap_lookup["n"],
-                    )
-                vmin, vmax = (np.real(np.min(field)), np.real(np.max(field)))
-                if show_sources:
-                    srcs = np.real(self.get_source_visual(field.shape))
-                    field = np.where(srcs, -max(np.abs(vmax), np.abs(vmin)) * 1000, field)
-                im = ax.imshow(
-                    np.real(field),
-                    extent=[np.real(z[0]), np.real(z[-1]), np.real(y[0]), np.real(y[-1])],
-                    cmap=cmap_lookup[component],
-                    alpha=1 if not show_geometry else 0.85,
-                )
-                ax.set_xlabel(np.real(axes[1]))
-                ax.set_ylabel(np.real(axes[0]))
-                ax.set_title(component)
-            else:
-                if show_geometry:
-                    plt.imshow(
-                        np.real(n),
-                        extent=[np.real(zn[0]), np.real(zn[-1]), np.real(yn[0]), np.real(yn[-1])],
-                        cmap=cmap_lookup["n"],
-                    )
-                vmin, vmax = (np.real(np.min(field)), np.real(np.max(field)))
-                if show_sources:
-                    srcs = np.real(self.get_source_visual(field.shape))
-                    field = np.where(srcs, -max(np.abs(vmax), np.abs(vmin)) * 1000, field)
-                im = plt.imshow(
-                    np.real(field),
-                    extent=[np.real(z[0]), np.real(z[-1]), np.real(y[0]), np.real(y[-1])],
-                    cmap=cmap_lookup[component],
-                    alpha=1 if not show_geometry else 0.85,
-                    vmin=vmin,
-                    vmax=vmax,
+
+            # Underlay the geometry
+            show = ax if ax else plt
+            if show_geometry:
+                show.imshow(
+                    np.real(n[::-1]),
+                    extent=[np.real(zn[0]), np.real(zn[-1]), np.real(yn[0]), np.real(yn[-1])],
+                    cmap=cmap_lookup["n"],
                 )
+            vmin, vmax = (np.real(np.min(field)), np.real(np.max(field)))
+            alpha = 1 if not show_geometry else 0.85
+
+            # Plot sources
+            if show_sources:
+                srcs = self.get_source_visual(np.real(y[0]), np.real(y[-1]))
+                for srcx, srcy in srcs:
+                    plt.plot(srcx, srcy, color="red", linewidth=2)
+
+            # Plot xy monitors
+            if show_xy_monitors:
+                xy_monitors = self.get_xy_monitor_visual(np.real(y[0]), np.real(y[-1]))
+                for locx, locy in xy_monitors:
+                    plt.plot(locx, locy, color="blue", linewidth=2)
+
+            im = show.imshow(
+                np.real(field[::-1]),
+                extent=[np.real(z[0]), np.real(z[-1]), np.real(y[0]), np.real(y[-1])],
+                cmap=cmap_lookup[component],
+                alpha=alpha,
+                vmin=vmin,
+                vmax=vmax,
+            )
+
+            # Assign labels
+            if not ax:
                 plt.xlabel(np.real(axes[1]))
                 plt.ylabel(np.real(axes[0]))
                 plt.title(component)
+            else:
+                ax.set_xlabel(np.real(axes[1]))
+                ax.set_ylabel(np.real(axes[0]))
+                ax.set_title(component)
+
+        # xy fields
         else:
             if ax:
                 im = ax.imshow(
-                    np.real(field).T,
+                    np.rot90(np.real(field)),
                     extent=[np.real(z[0]), np.real(z[-1]), np.real(y[0]), np.real(y[-1])],
                     cmap=cmap_lookup[component],
                 )
@@ -464,7 +487,7 @@ class Monitor(object):
                 ax.set_title(component)
             else:
                 im = plt.imshow(
-                    np.real(field).T,
+                    np.rot90(np.real(field)),
                     extent=[np.real(z[0]), np.real(z[-1]), np.real(y[0]), np.real(y[-1])],
                     cmap=cmap_lookup[component],
                 )
