@@ -1,11 +1,3 @@
-from abc import abstractmethod
-import numpy as np
-from tqdm import tqdm
-from simphony.models import Subcircuit
-from emepy.monitors import Monitor
-from copy import deepcopy
-
-
 class Source(object):
     """This class defines mode sources that can be created for monitors"""
 
