@@ -1,24 +1,16 @@
 import numpy as np
-import scipy
 import os
 from scipy.interpolate import griddata
 from scipy.sparse import coo_matrix
-from scipy.signal import resample
 import EMpy_gpu
 import collections
-from matplotlib import pyplot as plt
 from emepy.materials import *
 from typing import Callable
 from shapely.geometry import Polygon, Point
 
 
 def polygon_to_n_2D(
-    polygon: "Polygon",
-    x: list,
-    y: list,
-    subpixel=True,
-    core_index=3.4,
-    cladding_index=1.4,
+    polygon: "Polygon", x: list, y: list, subpixel=True, core_index=3.4, cladding_index=1.4
 ) -> "np.ndarray":
     # Create grid
     xx, yy = np.meshgrid(x, y)
@@ -66,14 +58,7 @@ def vertices_to_n(vertices: list, x: list, y: list, subpixel=True, core_index=3.
 
 
 def rectangle_to_n(
-    center: tuple,
-    width: float,
-    thickness: float,
-    x: list,
-    y: list,
-    subpixel=True,
-    core_index=3.4,
-    cladding_index=1.4,
+    center: tuple, width: float, thickness: float, x: list, y: list, subpixel=True, core_index=3.4, cladding_index=1.4
 ) -> "np.ndarray":
 
     xc, yc = center
@@ -87,66 +72,97 @@ def rectangle_to_n(
 
 
 def circle_to_n(
-    center: tuple,
-    radius: float,
-    x: list,
-    y: list,
-    subpixel=True,
-    core_index=3.4,
-    cladding_index=1.4,
+    center: tuple, radius: float, x: list, y: list, subpixel=True, core_index=3.4, cladding_index=1.4
 ) -> "np.ndarray":
 
     polygon = Point(*center).buffer(radius)
     return polygon_to_n_2D(polygon, x, y, subpixel, core_index, cladding_index)
 
 
-def get_epsfunc(
-    width: float,
-    thickness: float,
-    cladding_width: float,
-    cladding_thickness: float,
-    core_index: float,
-    cladding_index: float,
-    compute: bool = False,
-    profile: "np.ndarray" = None,
-    nx: int = None,
-    ny: int = None,
-) -> Callable[["np.ndarray", "np.ndarray"], "np.ndarray"]:
-    """Returns the epsfunc for given parameters for a rectangular waveguide
+class get_epsfunc(object):
+    """Callable class for getting epsilon on a grid"""
 
-    Parameters
-    ----------
-    width: float
-        the width of the geometry
-    thickness: float
-        the thickess of the geometry
-    cladding_width: float
-        the width of the surrounding cladding (note this is total width and should be > core width)
-    cladding_thickness: float
-        the thickness of the surrounding cladding (note this is total thickness and should be > core thickness)
-    core_index: float
-        refractive index of the core
-    cladding_index: float
-        refractive index of the cladding
-    compute: bool = False
-        if true, will not place rectangle at center. This should only be necessary for compute_other_fields
-    profile: "np.ndarray" = None
-        the refractive index profile (note if providing a width, this should be left None)
-    nx: int = None
-        number of points in the x direction
-    ny: int = None
-        number of points in the y direction
-    subpixel: bool = True
-        if true, will use subpixel smoothing, assuming asking for a waveguide cross section and not providing an index map (recommended)
+    def __init__(
+        self,
+        width: float,
+        thickness: float,
+        cladding_width: float,
+        cladding_thickness: float,
+        core_index: float,
+        cladding_index: float,
+        compute: bool = False,
+        profile: "np.ndarray" = None,
+        nx: int = None,
+        ny: int = None,
+    ):
+        """Returns the epsfunc for given parameters for a rectangular waveguide
 
-    Returns
-    -------
-    function
-        an epsfunc function that takes an x,y grid arrays and returns the refractive index profile
-    """
+        Parameters
+        ----------
+        width: float
+            the width of the geometry
+        thickness: float
+            the thickess of the geometry
+        cladding_width: float
+            the width of the surrounding cladding (note this is total width and should be > core width)
+        cladding_thickness: float
+            the thickness of the surrounding cladding (note this is total thickness and should be > core thickness)
+        core_index: float
+            refractive index of the core
+        cladding_index: float
+            refractive index of the cladding
+        compute: bool = False
+            if true, will not place rectangle at center. This should only be necessary for compute_other_fields
+        profile: "np.ndarray" = None
+            the refractive index profile (note if providing a width, this should be left None)
+        nx: int = None
+            number of points in the x direction
+        ny: int = None
+            number of points in the y direction
+        subpixel: bool = True
+            if true, will use subpixel smoothing, assuming asking for a waveguide cross section and not providing an index map (recommended)
+
+        Returns
+        -------
+        function
+            an epsfunc function that takes an x,y grid arrays and returns the refractive index profile
+        """
+        self.width = width
+        self.thickness = thickness
+        self.cladding = cladding_width
+        self.cladding = cladding_thickness
+        self.core_index = core_index
+        self.cladding_index = cladding_index
+        self.compute = compute
+        self.profile = profile
+        self.nx = nx
+        self.ny = ny
+
+        if (width is not None) and (thickness is not None):
+            self.epsfunction = self.epsfunc_2D_1
+
+        elif (width is None) and (thickness is not None) and (profile is not None):
+            self.epsfunction = self.epsfunc_2D_2
+
+        elif (width is None) and (thickness is None) and (profile is not None):
+            self.epsfunction = self.epsfunc_2D_3
+
+        elif (thickness is None) and (width is not None):
+            self.epsfunction = self.epsfunc_1D_1
+
+        elif (thickness is None) and (width is None) and (profile is not None):
+            self.epsfunction = self.epsfunc_1D_2
+
+        else:
+            raise Exception(
+                "Need to provide width & thickness, or 1D profile and thickness, or 2D profile, or width for 1D, or 1D profile for 1D"
+            )
+
+    def __call__(self, x_, y_):
+        return self.epsfunction(x_, y_)
 
     # Case 1 : width and thickness are defined
-    def epsfunc_2D_1(x_, y_):
+    def epsfunc_2D_1(self, x_, y_):
         """Return a matrix describing a 2d material.
 
         Parameters
@@ -162,37 +178,37 @@ def get_epsfunc(
             2d-matrix
         """
         xx, yy = np.meshgrid(x_, y_)
-        if compute:
+        if self.compute:
             n = np.where(
-                (np.abs(np.real(xx.T) - cladding_width * 0.5) <= width * 0.5)
-                * (np.abs(np.real(yy.T) - cladding_thickness * 0.5) <= thickness * 0.5),
-                core_index**2 + 0j,
-                cladding_index**2 + 0j,
+                (np.abs(np.real(xx.T) - self.cladding_width * 0.5) <= self.width * 0.5)
+                * (np.abs(np.real(yy.T) - self.cladding_thickness * 0.5) <= self.thickness * 0.5),
+                self.core_index ** 2 + 0j,
+                self.cladding_index ** 2 + 0j,
             )
         else:
             n = np.where(
-                (np.abs(np.real(xx.T)) <= width * 0.5) * (np.abs(np.real(yy.T)) <= thickness * 0.5),
-                core_index**2 + 0j,
-                cladding_index**2 + 0j,
+                (np.abs(np.real(xx.T)) <= self.width * 0.5) * (np.abs(np.real(yy.T)) <= self.thickness * 0.5),
+                self.core_index ** 2 + 0j,
+                self.cladding_index ** 2 + 0j,
             )
         return n
 
     # Case 2 : thickness and 1D n is defined
-    def epsfunc_2D_2(x_, y_):
+    def epsfunc_2D_2(self, x_, y_):
 
-        n = profile
+        n = self.profile
         xx, yy = np.meshgrid(np.real(x_), np.real(y_))
-        n = np.interp(np.real(x_), np.real(nx), n).astype(complex)
+        n = np.interp(np.real(x_), np.real(self.nx), n).astype(complex)
         n = np.repeat(n, len(y_)).reshape((len(n), len(y_)))
-        n = np.where((np.abs(np.real(yy.T)) <= thickness * 0.5), n, cladding_index + 0j) ** 2
+        n = np.where((np.abs(np.real(yy.T)) <= self.thickness * 0.5), n, self.cladding_index + 0j) ** 2
         return n
 
     # Case 3 : 2D n is defined
-    def epsfunc_2D_3(x_, y_):
+    def epsfunc_2D_3(self, x_, y_):
 
-        xxn, yyn = np.meshgrid(np.real(nx), np.real(ny))
+        xxn, yyn = np.meshgrid(np.real(self.nx), np.real(self.ny))
         points = np.array((xxn.flatten(), yyn.flatten())).T
-        n = profile.flatten()
+        n = self.profile.flatten()
         xx, yy = np.meshgrid(np.real(x_), np.real(y_))
         n_real = griddata(points, np.real(n), (xx, yy))
         n_imag = griddata(points, np.imag(n), (xx, yy))
@@ -200,43 +216,22 @@ def get_epsfunc(
         return n
 
     # Case 4: width only
-    def epsfunc_1D_1(x_, y_):
+    def epsfunc_1D_1(self, x_, y_):
 
         n = np.where(
-            (np.abs(np.real(x_)) <= width * 0.5),
-            core_index**2 + 0j,
-            cladding_index**2 + 0j,
+            (np.abs(np.real(x_)) <= self.width * 0.5), self.core_index ** 2 + 0j, self.cladding_index ** 2 + 0j
         ).reshape(len(x_), 1)
 
         return n
 
     # Case 5: 1D n only
-    def epsfunc_1D_2(x_, y_):
+    def epsfunc_1D_2(self, x_, y_):
 
-        n = np.interp(np.real(x_), np.real(nx), profile).astype(complex).reshape(len(x_), 1)
+        n = np.interp(np.real(x_), np.real(self.nx), self.profile).astype(complex).reshape(len(x_), 1)
         return n
 
-    if (width is not None) and (thickness is not None):
-        return epsfunc_2D_1
 
-    elif (width is None) and (thickness is not None) and (profile is not None):
-        return epsfunc_2D_2
-
-    elif (width is None) and (thickness is None) and (profile is not None):
-        return epsfunc_2D_3
-
-    elif (thickness is None) and (width is not None):
-        return epsfunc_1D_1
-
-    elif (thickness is None) and (width is None) and (profile is not None):
-        return epsfunc_1D_2
-
-    raise Exception(
-        "Need to provide width & thickness, or 1D profile and thickness, or 2D profile, or width for 1D, or 1D profile for 1D"
-    )
-
-
-def get_epsfunc_epsfunc(
+def get_isotropic_epsfunc(
     epsfunc_xx, epsfunc_yy, epsfunc_zz, epsfunc_xy=None, epsfunc_yx=None
 ) -> Callable[["np.ndarray", "np.ndarray"], "np.ndarray"]:
     """Returns an epsfunction for an isotropic medium"""
@@ -304,12 +299,7 @@ def create_polygon(x: "np.ndarray", y: "np.ndarray", n: "np.ndarray", detranslat
 
 
 def interp(
-    x: "np.ndarray",
-    y: "np.ndarray",
-    x0: "np.ndarray",
-    y0: "np.ndarray",
-    f: "np.ndarray",
-    centered: bool,
+    x: "np.ndarray", y: "np.ndarray", x0: "np.ndarray", y0: "np.ndarray", f: "np.ndarray", centered: bool
 ) -> "np.ndarray":
     """Interpolate a 2D complex array.
 
@@ -427,9 +417,7 @@ def from_chunks(location: str, name: str) -> None:
 
 
 def _get_eps(
-    xc: "np.ndarray",
-    yc: "np.ndarray",
-    epsfunc: Callable[["np.ndarray", "np.ndarray"], "np.ndarray"],
+    xc: "np.ndarray", yc: "np.ndarray", epsfunc: Callable[["np.ndarray", "np.ndarray"], "np.ndarray"]
 ) -> tuple:
     """Used by compute_other_fields and adapted from the EMpy library"""
     tmp = epsfunc(xc, yc)
@@ -666,14 +654,14 @@ def compute_other_fields_2D(
             * ezz1
             * ezz2
             / eyy1
-            * (2 * eyy1 / ezz1 / n**2 + eyx1 / ezz1 / n / w)
+            * (2 * eyy1 / ezz1 / n ** 2 + eyx1 / ezz1 / n / w)
             + 0.5
             * (n * ezz1 * ezz2 / eyy1 + s * ezz2 * ezz1 / eyy2)
             * n
             * ezz4
             * ezz3
             / eyy4
-            * (2 * eyy4 / ezz4 / n**2 - eyx4 / ezz4 / n / e)
+            * (2 * eyy4 / ezz4 / n ** 2 - eyx4 / ezz4 / n / e)
         )
         / ezz4
         / ezz3
@@ -693,14 +681,14 @@ def compute_other_fields_2D(
             * (
                 0.5
                 * ezz4
-                * ((1 - exx1 / ezz1) / n / w - exy1 / ezz1 * (2.0 / n**2 - 2 / n**2 * s / (n + s)))
+                * ((1 - exx1 / ezz1) / n / w - exy1 / ezz1 * (2.0 / n ** 2 - 2 / n ** 2 * s / (n + s)))
                 / exx1
                 * ezz1
                 * w
                 + (ezz4 - ezz1) * s / n / (n + s)
                 + 0.5
                 * ezz1
-                * (-(1 - exx4 / ezz4) / n / e - exy4 / ezz4 * (2.0 / n**2 - 2 / n**2 * s / (n + s)))
+                * (-(1 - exx4 / ezz4) / n / e - exy4 / ezz4 * (2.0 / n ** 2 - 2 / n ** 2 * s / (n + s)))
                 / exx4
                 * ezz4
                 * e
@@ -735,14 +723,14 @@ def compute_other_fields_2D(
             * ezz2
             * ezz1
             / eyy2
-            * (2 * eyy2 / ezz2 / s**2 - eyx2 / ezz2 / s / w)
+            * (2 * eyy2 / ezz2 / s ** 2 - eyx2 / ezz2 / s / w)
             + 0.5
             * (n * ezz1 * ezz2 / eyy1 + s * ezz2 * ezz1 / eyy2)
             * s
             * ezz3
             * ezz4
             / eyy3
-            * (2 * eyy3 / ezz3 / s**2 + eyx3 / ezz3 / s / e)
+            * (2 * eyy3 / ezz3 / s ** 2 + eyx3 / ezz3 / s / e)
         )
         / ezz4
         / ezz3
@@ -768,14 +756,14 @@ def compute_other_fields_2D(
             * (
                 0.5
                 * ezz3
-                * (-(1 - exx2 / ezz2) / s / w - exy2 / ezz2 * (2.0 / s**2 - 2 / s**2 * n / (n + s)))
+                * (-(1 - exx2 / ezz2) / s / w - exy2 / ezz2 * (2.0 / s ** 2 - 2 / s ** 2 * n / (n + s)))
                 / exx2
                 * ezz2
                 * w
                 - (ezz3 - ezz2) * n / s / (n + s)
                 + 0.5
                 * ezz2
-                * ((1 - exx3 / ezz3) / s / e - exy3 / ezz3 * (2.0 / s**2 - 2 / s**2 * n / (n + s)))
+                * ((1 - exx3 / ezz3) / s / e - exy3 / ezz3 * (2.0 / s ** 2 - 2 / s ** 2 * n / (n + s)))
                 / exx3
                 * ezz3
                 * e
@@ -799,8 +787,8 @@ def compute_other_fields_2D(
     bzxe = (
         (n * ezz1 * ezz2 / eyy1 + s * ezz2 * ezz1 / eyy2)
         * (
-            0.5 * n * ezz4 * ezz3 / eyy4 * (2.0 / e**2 - eyx4 / ezz4 / n / e)
-            + 0.5 * s * ezz3 * ezz4 / eyy3 * (2.0 / e**2 + eyx3 / ezz3 / s / e)
+            0.5 * n * ezz4 * ezz3 / eyy4 * (2.0 / e ** 2 - eyx4 / ezz4 / n / e)
+            + 0.5 * s * ezz3 * ezz4 / eyy3 * (2.0 / e ** 2 + eyx3 / ezz3 / s / e)
         )
         / ezz4
         / ezz3
@@ -837,8 +825,8 @@ def compute_other_fields_2D(
     bzxw = (
         (-n * ezz4 * ezz3 / eyy4 - s * ezz3 * ezz4 / eyy3)
         * (
-            0.5 * n * ezz1 * ezz2 / eyy1 * (2.0 / w**2 + eyx1 / ezz1 / n / w)
-            + 0.5 * s * ezz2 * ezz1 / eyy2 * (2.0 / w**2 - eyx2 / ezz2 / s / w)
+            0.5 * n * ezz1 * ezz2 / eyy1 * (2.0 / w ** 2 + eyx1 / ezz1 / n / w)
+            + 0.5 * s * ezz2 * ezz1 / eyy2 * (2.0 / w ** 2 - eyx2 / ezz2 / s / w)
         )
         / ezz4
         / ezz3
@@ -881,13 +869,13 @@ def compute_other_fields_2D(
                 * ezz1
                 * ezz2
                 / eyy1
-                * (-2.0 / w**2 - 2 * eyy1 / ezz1 / n**2 + k**2 * eyy1 - eyx1 / ezz1 / n / w)
+                * (-2.0 / w ** 2 - 2 * eyy1 / ezz1 / n ** 2 + k ** 2 * eyy1 - eyx1 / ezz1 / n / w)
                 + 0.5
                 * s
                 * ezz2
                 * ezz1
                 / eyy2
-                * (-2.0 / w**2 - 2 * eyy2 / ezz2 / s**2 + k**2 * eyy2 + eyx2 / ezz2 / s / w)
+                * (-2.0 / w ** 2 - 2 * eyy2 / ezz2 / s ** 2 + k ** 2 * eyy2 + eyx2 / ezz2 / s / w)
             )
             + (n * ezz1 * ezz2 / eyy1 + s * ezz2 * ezz1 / eyy2)
             * (
@@ -896,13 +884,13 @@ def compute_other_fields_2D(
                 * ezz4
                 * ezz3
                 / eyy4
-                * (-2.0 / e**2 - 2 * eyy4 / ezz4 / n**2 + k**2 * eyy4 + eyx4 / ezz4 / n / e)
+                * (-2.0 / e ** 2 - 2 * eyy4 / ezz4 / n ** 2 + k ** 2 * eyy4 + eyx4 / ezz4 / n / e)
                 + 0.5
                 * s
                 * ezz3
                 * ezz4
                 / eyy3
-                * (-2.0 / e**2 - 2 * eyy3 / ezz3 / s**2 + k**2 * eyy3 - eyx3 / ezz3 / s / e)
+                * (-2.0 / e ** 2 - 2 * eyy3 / ezz3 / s ** 2 + k ** 2 * eyy3 - eyx3 / ezz3 / s / e)
             )
         )
         / ezz4
@@ -924,9 +912,9 @@ def compute_other_fields_2D(
                 0.5
                 * ezz4
                 * (
-                    -(k**2) * exy1
+                    -(k ** 2) * exy1
                     - (1 - exx1 / ezz1) / n / w
-                    - exy1 / ezz1 * (-2.0 / n**2 - 2 / n**2 * (n - s) / s)
+                    - exy1 / ezz1 * (-2.0 / n ** 2 - 2 / n ** 2 * (n - s) / s)
                 )
                 / exx1
                 * ezz1
@@ -935,9 +923,9 @@ def compute_other_fields_2D(
                 + 0.5
                 * ezz1
                 * (
-                    -(k**2) * exy4
+                    -(k ** 2) * exy4
                     + (1 - exx4 / ezz4) / n / e
-                    - exy4 / ezz4 * (-2.0 / n**2 - 2 / n**2 * (n - s) / s)
+                    - exy4 / ezz4 * (-2.0 / n ** 2 - 2 / n ** 2 * (n - s) / s)
                 )
                 / exx4
                 * ezz4
@@ -948,9 +936,9 @@ def compute_other_fields_2D(
                 0.5
                 * ezz3
                 * (
-                    -(k**2) * exy2
+                    -(k ** 2) * exy2
                     + (1 - exx2 / ezz2) / s / w
-                    - exy2 / ezz2 * (-2.0 / s**2 + 2 / s**2 * (n - s) / n)
+                    - exy2 / ezz2 * (-2.0 / s ** 2 + 2 / s ** 2 * (n - s) / n)
                 )
                 / exx2
                 * ezz2
@@ -959,9 +947,9 @@ def compute_other_fields_2D(
                 + 0.5
                 * ezz2
                 * (
-                    -(k**2) * exy3
+                    -(k ** 2) * exy3
                     - (1 - exx3 / ezz3) / s / e
-                    - exy3 / ezz3 * (-2.0 / s**2 + 2 / s**2 * (n - s) / n)
+                    - exy3 / ezz3 * (-2.0 / s ** 2 + 2 / s ** 2 * (n - s) / n)
                 )
                 / exx3
                 * ezz3
@@ -1115,8 +1103,8 @@ def compute_other_fields_2D(
         * e
         + (ezz3 / exx2 * ezz2 * w + ezz2 / exx3 * ezz3 * e)
         * (
-            0.5 * ezz4 * (2.0 / n**2 + exy1 / ezz1 / n / w) / exx1 * ezz1 * w
-            + 0.5 * ezz1 * (2.0 / n**2 - exy4 / ezz4 / n / e) / exx4 * ezz4 * e
+            0.5 * ezz4 * (2.0 / n ** 2 + exy1 / ezz1 / n / w) / exx1 * ezz1 * w
+            + 0.5 * ezz1 * (2.0 / n ** 2 - exy4 / ezz4 / n / e) / exx4 * ezz4 * e
         )
         / ezz3
         / ezz2
@@ -1153,8 +1141,8 @@ def compute_other_fields_2D(
         * e
         - (ezz4 / exx1 * ezz1 * w + ezz1 / exx4 * ezz4 * e)
         * (
-            0.5 * ezz3 * (2.0 / s**2 - exy2 / ezz2 / s / w) / exx2 * ezz2 * w
-            + 0.5 * ezz2 * (2.0 / s**2 + exy3 / ezz3 / s / e) / exx3 * ezz3 * e
+            0.5 * ezz3 * (2.0 / s ** 2 - exy2 / ezz2 / s / w) / exx2 * ezz2 * w
+            + 0.5 * ezz2 * (2.0 / s ** 2 + exy3 / ezz3 / s / e) / exx3 * ezz3 * e
         )
         / ezz3
         / ezz2
@@ -1186,13 +1174,13 @@ def compute_other_fields_2D(
                 * ezz4
                 * ezz3
                 / eyy4
-                * (-(1 - eyy4 / ezz4) / n / e - eyx4 / ezz4 * (2.0 / e**2 - 2 / e**2 * w / (e + w)))
+                * (-(1 - eyy4 / ezz4) / n / e - eyx4 / ezz4 * (2.0 / e ** 2 - 2 / e ** 2 * w / (e + w)))
                 + 0.5
                 * s
                 * ezz3
                 * ezz4
                 / eyy3
-                * ((1 - eyy3 / ezz3) / s / e - eyx3 / ezz3 * (2.0 / e**2 - 2 / e**2 * w / (e + w)))
+                * ((1 - eyy3 / ezz3) / s / e - eyx3 / ezz3 * (2.0 / e ** 2 - 2 / e ** 2 * w / (e + w)))
                 + (ezz4 - ezz3) * w / e / (e + w)
             )
         )
@@ -1213,14 +1201,14 @@ def compute_other_fields_2D(
             0.5
             * (ezz3 / exx2 * ezz2 * w + ezz2 / exx3 * ezz3 * e)
             * ezz1
-            * (2 * exx4 / ezz4 / e**2 - exy4 / ezz4 / n / e)
+            * (2 * exx4 / ezz4 / e ** 2 - exy4 / ezz4 / n / e)
             / exx4
             * ezz4
             * e
             - 0.5
             * (ezz4 / exx1 * ezz1 * w + ezz1 / exx4 * ezz4 * e)
             * ezz2
-            * (2 * exx3 / ezz3 / e**2 + exy3 / ezz3 / s / e)
+            * (2 * exx3 / ezz3 / e ** 2 + exy3 / ezz3 / s / e)
             / exx3
             * ezz3
             * e
@@ -1249,14 +1237,14 @@ def compute_other_fields_2D(
                 * ezz1
                 * ezz2
                 / eyy1
-                * ((1 - eyy1 / ezz1) / n / w - eyx1 / ezz1 * (2.0 / w**2 - 2 / w**2 * e / (e + w)))
+                * ((1 - eyy1 / ezz1) / n / w - eyx1 / ezz1 * (2.0 / w ** 2 - 2 / w ** 2 * e / (e + w)))
                 - (ezz1 - ezz2) * e / w / (e + w)
                 + 0.5
                 * s
                 * ezz2
                 * ezz1
                 / eyy2
-                * (-(1 - eyy2 / ezz2) / s / w - eyx2 / ezz2 * (2.0 / w**2 - 2 / w**2 * e / (e + w)))
+                * (-(1 - eyy2 / ezz2) / s / w - eyx2 / ezz2 * (2.0 / w ** 2 - 2 / w ** 2 * e / (e + w)))
             )
             + (n * ezz1 * ezz2 / eyy1 + s * ezz2 * ezz1 / eyy2)
             * (
@@ -1282,14 +1270,14 @@ def compute_other_fields_2D(
             0.5
             * (ezz3 / exx2 * ezz2 * w + ezz2 / exx3 * ezz3 * e)
             * ezz4
-            * (2 * exx1 / ezz1 / w**2 + exy1 / ezz1 / n / w)
+            * (2 * exx1 / ezz1 / w ** 2 + exy1 / ezz1 / n / w)
             / exx1
             * ezz1
             * w
             - 0.5
             * (ezz4 / exx1 * ezz1 * w + ezz1 / exx4 * ezz4 * e)
             * ezz3
-            * (2 * exx2 / ezz2 / w**2 - exy2 / ezz2 / s / w)
+            * (2 * exx2 / ezz2 / w ** 2 - exy2 / ezz2 / s / w)
             / exx2
             * ezz2
             * w
@@ -1319,9 +1307,9 @@ def compute_other_fields_2D(
                 * ezz2
                 / eyy1
                 * (
-                    -(k**2) * eyx1
+                    -(k ** 2) * eyx1
                     - (1 - eyy1 / ezz1) / n / w
-                    - eyx1 / ezz1 * (-2.0 / w**2 + 2 / w**2 * (e - w) / e)
+                    - eyx1 / ezz1 * (-2.0 / w ** 2 + 2 / w ** 2 * (e - w) / e)
                 )
                 + (ezz1 - ezz2) * (e - w) / e / w
                 + 0.5
@@ -1330,9 +1318,9 @@ def compute_other_fields_2D(
                 * ezz1
                 / eyy2
                 * (
-                    -(k**2) * eyx2
+                    -(k ** 2) * eyx2
                     + (1 - eyy2 / ezz2) / s / w
-                    - eyx2 / ezz2 * (-2.0 / w**2 + 2 / w**2 * (e - w) / e)
+                    - eyx2 / ezz2 * (-2.0 / w ** 2 + 2 / w ** 2 * (e - w) / e)
                 )
             )
             + (n * ezz1 * ezz2 / eyy1 + s * ezz2 * ezz1 / eyy2)
@@ -1343,9 +1331,9 @@ def compute_other_fields_2D(
                 * ezz3
                 / eyy4
                 * (
-                    -(k**2) * eyx4
+                    -(k ** 2) * eyx4
                     + (1 - eyy4 / ezz4) / n / e
-                    - eyx4 / ezz4 * (-2.0 / e**2 - 2 / e**2 * (e - w) / w)
+                    - eyx4 / ezz4 * (-2.0 / e ** 2 - 2 / e ** 2 * (e - w) / w)
                 )
                 + 0.5
                 * s
@@ -1353,9 +1341,9 @@ def compute_other_fields_2D(
                 * ezz4
                 / eyy3
                 * (
-                    -(k**2) * eyx3
+                    -(k ** 2) * eyx3
                     - (1 - eyy3 / ezz3) / s / e
-                    - eyx3 / ezz3 * (-2.0 / e**2 - 2 / e**2 * (e - w) / w)
+                    - eyx3 / ezz3 * (-2.0 / e ** 2 - 2 / e ** 2 * (e - w) / w)
                 )
                 + (ezz4 - ezz3) * (e - w) / e / w
             )
@@ -1378,13 +1366,13 @@ def compute_other_fields_2D(
             * (
                 0.5
                 * ezz4
-                * (-2.0 / n**2 - 2 * exx1 / ezz1 / w**2 + k**2 * exx1 - exy1 / ezz1 / n / w)
+                * (-2.0 / n ** 2 - 2 * exx1 / ezz1 / w ** 2 + k ** 2 * exx1 - exy1 / ezz1 / n / w)
                 / exx1
                 * ezz1
                 * w
                 + 0.5
                 * ezz1
-                * (-2.0 / n**2 - 2 * exx4 / ezz4 / e**2 + k**2 * exx4 + exy4 / ezz4 / n / e)
+                * (-2.0 / n ** 2 - 2 * exx4 / ezz4 / e ** 2 + k ** 2 * exx4 + exy4 / ezz4 / n / e)
                 / exx4
                 * ezz4
                 * e
@@ -1393,13 +1381,13 @@ def compute_other_fields_2D(
             * (
                 0.5
                 * ezz3
-                * (-2.0 / s**2 - 2 * exx2 / ezz2 / w**2 + k**2 * exx2 + exy2 / ezz2 / s / w)
+                * (-2.0 / s ** 2 - 2 * exx2 / ezz2 / w ** 2 + k ** 2 * exx2 + exy2 / ezz2 / s / w)
                 / exx2
                 * ezz2
                 * w
                 + 0.5
                 * ezz2
-                * (-2.0 / s**2 - 2 * exx3 / ezz3 / e**2 + k**2 * exx3 - exy3 / ezz3 / s / e)
+                * (-2.0 / s ** 2 - 2 * exx3 / ezz3 / e ** 2 + k ** 2 * exx3 - exy3 / ezz3 / s / e)
                 / exx3
                 * ezz3
                 * e
@@ -1517,29 +1505,13 @@ def compute_other_fields_2D(
     Izx = np.r_[iall, i_w, i_e, i_s, i_n, i_ne, i_se, i_sw, i_nw]
     Jzx = np.r_[iall, i_e, i_w, i_n, i_s, i_sw, i_nw, i_ne, i_se]
     Vzx = np.r_[
-        bzxp[iall],
-        bzxe[i_w],
-        bzxw[i_e],
-        bzxn[i_s],
-        bzxs[i_n],
-        bzxsw[i_ne],
-        bzxnw[i_se],
-        bzxne[i_sw],
-        bzxse[i_nw],
+        bzxp[iall], bzxe[i_w], bzxw[i_e], bzxn[i_s], bzxs[i_n], bzxsw[i_ne], bzxnw[i_se], bzxne[i_sw], bzxse[i_nw]
     ]
 
     Izy = np.r_[iall, i_w, i_e, i_s, i_n, i_ne, i_se, i_sw, i_nw]
     Jzy = np.r_[iall, i_e, i_w, i_n, i_s, i_sw, i_nw, i_ne, i_se] + nx * ny
     Vzy = np.r_[
-        bzyp[iall],
-        bzye[i_w],
-        bzyw[i_e],
-        bzyn[i_s],
-        bzys[i_n],
-        bzysw[i_ne],
-        bzynw[i_se],
-        bzyne[i_sw],
-        bzyse[i_nw],
+        bzyp[iall], bzye[i_w], bzyw[i_e], bzyn[i_s], bzys[i_n], bzysw[i_ne], bzynw[i_se], bzyne[i_sw], bzyse[i_nw]
     ]
 
     I = np.r_[Izx, Izy]
